package jp.programmingmat.www.gbtkgl10Sample01;

/**
	GbtkGL10ライブラリサンプルworld
*/
import jp.programmingmat.www.gbtkgl10.*;

import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.Paint.Style;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.os.Handler;

public class SampleWorld extends GbtkGL10World {

	private GbtkGL10Mesh mEarth;
	private GbtkGL10DrawableObjectDuplicator mEarthList;
	
	private GbtkGL10Mesh mEarthBase;
	
	private GbtkGL10Mesh mXYCylinder, mXZCylinder, mYZCylinder;
	
	private GbtkGL10Vector3f[] mEarthRotationArray;
	private GbtkGL10Object mSampleObject;

	private int mProcessTick;
	private int mProcessTotalMS;
	private long mPrevProcessMS;
	
	private double mFrameMS;

	private Handler mUIHandler;
	
	public SampleWorld(GbtkGL10Activity activity) {
		
		super(activity);

		mUIHandler = new Handler();
		
	}

	@Override
	public void init() {

		// 土台を作成
		mEarthBase = new GbtkGL10Mesh(GbtkGL10Geometry.createBoxGeometry(20, 4, 80), new GbtkGL10Material(0.8f, 0.65f, 0.32f, 1, 1));
		mEarthBase.setPosition(0, -6, -52);
		
		addDrawObject(mEarthBase);

		// 地球オブジェクトのmaterial作成
		GbtkGL10Material earthMaterial = new GbtkGL10Material();

		// テクスチャ0番を使用するよう設定
		earthMaterial.setTexture(0);

		// 球体の地球オブジェクトを作成
		mEarth = new GbtkGL10Mesh(new GbtkGL10Geometry(GbtkGL10Element.createYSphereElement(4, 32)), earthMaterial);

		// 複数の球体を描画するObjectDuplicatorを作成
		mEarthList = new GbtkGL10DrawableObjectDuplicator(mEarth);

		mEarthRotationArray = new GbtkGL10Vector3f[8];

		for (int i = 0;i < 8;i++) {

			// ObjectDuplicatorに描画位置を設定
			if (i % 2 == 0) {
				mEarthList.addDrawTransform(-4.1f, 0, -16 -(i / 2) * 10);
			} else {
				mEarthList.addDrawTransform(4.1f, 0, -16 -(i / 2) * 10);
			}

			// 各球体の回転角度を保持する配列を設定
			mEarthRotationArray[i] = new GbtkGL10Vector3f((float)Math.random(), (float)Math.random(), (float)Math.random());
			
		}
		
		addDrawObject(mEarthList);

		// カメラ位置と角度を設定
		getCamera().setPosition(0, 12, 0);
		getCamera().setLookVector(0, -5.0f, -10.0f);

		mXYCylinder = new GbtkGL10Mesh(GbtkGL10Geometry.createXYCylinderGeometry(3, 3, 16, false), new GbtkGL10Material(0.5f, 0.5f, 0.5f, 1.0f));
		
		mXYCylinder.setPosition(-7, 6, -20);
		addDrawObject(mXYCylinder);

		mXZCylinder = new GbtkGL10Mesh(GbtkGL10Geometry.createXZCylinderGeometry(3, 3, 16, false), new GbtkGL10Material(0.15f, 0.15f, 0.1f, 1.0f));
		mXZCylinder.getMaterial().setSpecularRGBA(0.6f, 0.6f, 0.5f, 1);
		mXZCylinder.getMaterial().setShininess(63);
		
		mXZCylinder.setPosition(0, 6, -20);
		addDrawObject(mXZCylinder);

		mYZCylinder = new GbtkGL10Mesh(GbtkGL10Geometry.createYZCylinderGeometry(3, 3, 16, false), new GbtkGL10Material(1.0f, 0.5f, 0.25f, 1.0f));

		mYZCylinder.setPosition(7, 6, -20);
		addDrawObject(mYZCylinder);
		
		mSampleObject = new SampleObject();
		
		mSampleObject.setPosition(0, 10, -5);
		addDrawObject(mSampleObject);
		
		mProcessTotalMS = 0;
		mPrevProcessMS = getWorldTimeMillis();
		
	}

	@Override
	public void loadTexture() {

		Bitmap bmp = BitmapFactory.decodeResource(getResources(), R.drawable.earth512);

		setTexture(bmp, 0);

		bmp.recycle();
		
		bmp = Bitmap.createBitmap(256, 256, Config.ARGB_8888);
		
		Canvas cv = new Canvas(bmp);
		cv.drawARGB(255, 255, 192, 192);

		Paint pt = new Paint();
		pt.setStyle(Style.FILL);
		
		for (int i = 0;i < 128;i++) {

			pt.setARGB(255, 255 - i * 2, i * 2, i * 2);
			
			cv.drawRect(i, i, 255 - i, 255 - i, pt);

		}

		setTexture(bmp, 1);

		bmp.recycle();

		bmp = Bitmap.createBitmap(256, 256, Config.ARGB_8888);
		
		cv = new Canvas(bmp);
		cv.drawARGB(255, 192, 255, 192);

		pt = new Paint();
		pt.setStyle(Style.FILL);
		
		for (int i = 0;i < 128;i++) {

			pt.setARGB(255, 255 - i * 2, i * 2, i * 2);

			int x = (int)(Math.random() * 256);
			int y = (int)(Math.random() * 256);
			
			cv.drawCircle(x, y, 8, pt);

		}

		setTexture(bmp, 2);

		bmp.recycle();

		bmp = Bitmap.createBitmap(256, 256, Config.ARGB_8888);
		
		cv = new Canvas(bmp);
		cv.drawARGB(255, 192, 192, 255);

		pt = new Paint();
		pt.setStrokeWidth(2);
		pt.setARGB(255, 255, 255, 255);

		for (int i = 0;i < 32;i++) {
			
			cv.drawLine(0, i * 8, 255, i * 8, pt);
			cv.drawLine(i * 8, 0, i * 8, 255, pt);

		}
		
		setTexture(bmp, 3);
		
		bmp.recycle();
		
	}
	
	@Override
	public void process() {

		long now = getWorldTimeMillis();

		mProcessTotalMS += (int)(now - mPrevProcessMS);

		// 今回の処理開始時間を前回処理時間として記録
		mPrevProcessMS = now;

		// 球体の回転を更新
		for (int i = 0;i < mEarthList.length();i++) {
			mEarthList.getDrawTransform(i).addRotate(mEarthRotationArray[i].get(0) * (1.0f + (i / 4.0f)), mEarthRotationArray[i].get(0) * (1.0f + (i / 4.0f)), mEarthRotationArray[i].get(0) * (1.0f + (i / 4.0f)));
		}
	
		mXYCylinder.getMaterial().setEmissionRGBA((float)Math.random(), 0.15f, 0.5f, 1);
		mXYCylinder.addRotate(2, 0, 0);
		mXZCylinder.addRotate(0, 0, 2);
		mYZCylinder.addRotate(0, 2, 0);

		mSampleObject.move(0, 0, -0.06f);
		mSampleObject.addRotate(1, 0, 0);
		
		// 30フレーム毎に土台のテクスチャを変更、さらにフレームレートを算出してActivityに通知
		if ((mProcessTick % 30) == 29) {
			
			mEarthBase.getMaterial().setTexture(1 + (int)(Math.random() * 3));

			mFrameMS = mProcessTotalMS / 30.0;
			
			mUIHandler.post(new Runnable() {
				public void run() {
					((SampleActivity)getActivity()).updateFrameMS(mFrameMS);
				}
			});
			
			mProcessTotalMS = 0;
			
		}
		
		mProcessTick++;
		
	}

}