package jp.programmingmat.www.gbtkgl10;

/**
 * 3D座標やベクトル処理のための三次元ベクトルクラス。
 */
public class GbtkGL10Vector3f {

	private float[] mVector3f;
	
	public GbtkGL10Vector3f() {

		mVector3f = new float[]{0.0f, 0.0f, 0.0f};
		
	}

	public GbtkGL10Vector3f(float m0, float m1, float m2) {

		mVector3f = new float[]{m0, m1, m2};
		
	}

	/**
	 * 指定の配列からオブジェクトを作成
	 * @param data 要素数3以上の配列。
	 */
	public GbtkGL10Vector3f(float[] data) {

		if (data == null || data.length < 3) {
			mVector3f = new float[]{0.0f, 0.0f, 0.0f};
		} else {
			mVector3f = new float[]{data[0], data[1], data[2]};
		}
		
	}

	/**
	 * コピーコンストラクタ
	 * @param data
	 */
	public GbtkGL10Vector3f(GbtkGL10Vector3f data) {

		if (data == null || data.getFloatArray() == null) {
			mVector3f = new float[]{0.0f, 0.0f, 0.0f};
		} else {
			mVector3f = new float[]{data.mVector3f[0], data.mVector3f[1], data.mVector3f[2]};
		}

	}

	/**
	 * ベクトルの指定要素を取得
	 * @param index 取得する要素
	 * @return 指定位置にある要素
	 */
	public float get(int index) {

		if (index < 0 || index > 2) {
			return Float.NaN;
		}

		return mVector3f[index];
		
	}

	/**
	 * 保持しているベクトル配列（要素数3）を取得
	 * @return ベクトル要素を格納した配列
	 */
	public float[] getFloatArray() {
		return mVector3f;
	}

	public void set(GbtkGL10Vector3f vec) {

		if (vec == null) {
			set(0, 0, 0);
		} else {
			set(vec.get(0), vec.get(1), vec.get(2));
		}
		
	}
	
	public void set(float m0, float m1, float m2) {

		mVector3f[0] = m0;
		mVector3f[1] = m1;
		mVector3f[2] = m2;
		
	}

	public void add(GbtkGL10Vector3f vec) {

		if (vec == null) {
			return;
		}
		
		for (int i = 0;i < 3;i++) {
			mVector3f[i] += vec.mVector3f[i];
		}
		
	}
	
	public void add(float dm0, float dm1, float dm2) {

		mVector3f[0] += dm0;
		mVector3f[1] += dm1;
		mVector3f[2] += dm2;
		
	}
	
}